@@ -4,7 +4,7 @@ Utility functions for automatic mixed precision
 This module contains utility functions used by the AMP system for
 determining precision, casting tensors, and managing operation types.
 """
-
+import neurograd as ng
 from typing import Set
 from .autocast import autocast
 
@@ -16,6 +16,9 @@ _FP32_OPS: Set[str] = {
     "sin", "cos", "tan",
     # softmax & reductions that accumulate / are variance-based
     "softmax",
+    # norms / stats
+    "batchnorm", "batchnorm2d", "layernorm", "groupnorm", "instancenorm",
+    "var", "variance", "logsumexp",
     "sum", "mean", "std",
     # losses (compute entirely in fp32)
     "mse", "rmse", "mae", "binarycrossentropy", "categoricalcrossentropy",
@@ -36,7 +39,7 @@ _FP16_SAFE_OPS: Set[str] = {
     # padding and elementwise
     "pad", "abs", "clip", "max", "min",
     # activations (excluding softmax)
-    "relu", "leakyrelu", "sigmoid", "tanh", "passthrough",
+    "relu", "relu6", "leakyrelu", "sigmoid", "tanh", "passthrough",
 }
 
 
@@ -68,42 +71,55 @@ def should_cast_to_fp16(op_name: str) -> bool:
     if op_name_lower in _FP16_SAFE_OPS:
         return True
         
-    # Default behavior: allow FP16 for most ops but with caution
-    return True
+    # Default behavior: keep unknown ops in FP32 for safety
+    return False
 
 
-def maybe_cast_tensor(tensor, target_dtype=None, op_name: str = "unknown") -> 'Tensor':
+def maybe_cast_data(tensor, target_dtype=None, op_name: str = "unknown"):
     """
-    Cast tensor to appropriate dtype based on autocast context and operation type.
-    
-    Args:
-        tensor: Input tensor
-        target_dtype: Target dtype (if None, uses autocast dtype)
-        op_name: Name of the operation for casting decision
-        
-    Returns:
-        Tensor cast to appropriate dtype
+    Cast underlying array for compute without inserting Cast nodes.
+
+    Returns an xp.ndarray suitable for the op's forward, following autocast policy.
+    This mirrors PyTorch's behavior by selecting compute dtype per-op without
+    modifying the input Tensor's dtype or graph.
     """
     from neurograd.tensor import Tensor
-    
+
+    # If input isn't a Tensor or autocast disabled, just return raw data
     if not isinstance(tensor, Tensor):
         return tensor
-        
     if not autocast.is_enabled():
-        return tensor
-    
-    # Determine target dtype
+        return tensor.data
+
+    # Avoid recursion/casts for explicit Cast operations
+    if op_name and op_name.lower() == 'cast':
+        return tensor.data
+
+    # Decide target dtype
     if target_dtype is None:
         if should_cast_to_fp16(op_name):
             target_dtype = autocast.get_autocast_dtype()
         else:
-            return tensor.cast('float32')
-    
-    # Only cast if different from current dtype
+            # Force FP32 for sensitive ops
+            import neurograd as ng
+            target_dtype = ng.float32
+
+    # No-op if already correct dtype
     if tensor.data.dtype == target_dtype:
-        return tensor
-        
-    return tensor.cast(target_dtype)
+        return tensor.data
+
+    # Use per-context cache to reuse cast arrays within region
+    cache = autocast.get_cache()
+    if cache is not None:
+        key = ('data', id(tensor), target_dtype)
+        cached = cache.get(key)
+        if cached is not None:
+            return cached
+        arr = tensor.data.astype(target_dtype, copy=False)
+        cache[key] = arr
+        return arr
+
+    return tensor.data.astype(target_dtype, copy=False)
 
 
 def get_fp32_ops() -> Set[str]:
