@@ -86,7 +86,7 @@ class Tensor:
             
             # Compute gradients w.r.t. parent tensors
             parent_grads = tensor.grad_fn.backward(grad_output)
-            
+
             # Handle single gradient return (convert to tuple)
             if not isinstance(parent_grads, tuple):
                 parent_grads = (parent_grads,)
@@ -368,6 +368,10 @@ class Tensor:
         from .functions.activations import ReLU
         return ReLU()(self)
     
+    def relu6(self) -> 'Tensor':
+        from .functions.activations import ReLU6
+        return ReLU6()(self)
+    
     def sigmoid(self) -> 'Tensor':
         from .functions.activations import Sigmoid
         return Sigmoid()(self)
@@ -545,7 +549,7 @@ class Tensor:
         """Return a string representation of the tensor."""
         return f"Tensor({self.data})"
 
-
+# NON-DIFF
 def zeros(shape: Union[int, List[int]], dtype: Optional[str] = None,
           requires_grad: bool = False) -> Tensor:
     return Tensor(xp.zeros(shape, dtype=dtype), requires_grad=requires_grad)
